@@ -59,6 +59,7 @@ enum
   PROP_TURN_SERVER,
   PROP_CONTROLLER,
   PROP_AGENT,
+  PROP_FORCE_RELAY,
 };
 
 static guint gst_webrtc_ice_signals[LAST_SIGNAL] = { 0 };
@@ -79,8 +80,8 @@ struct _GstWebRTCICEPrivate
 #define gst_webrtc_ice_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstWebRTCICE, gst_webrtc_ice,
     GST_TYPE_OBJECT, G_ADD_PRIVATE (GstWebRTCICE)
-    GST_DEBUG_CATEGORY_INIT (gst_webrtc_ice_debug, "webrtcice", 0, "webrtcice");
-    );
+    GST_DEBUG_CATEGORY_INIT (gst_webrtc_ice_debug, "webrtcice", 0,
+        "webrtcice"););
 
 static gboolean
 _unlock_pc_thread (GMutex * lock)
@@ -826,6 +827,10 @@ gst_webrtc_ice_set_property (GObject * object, guint prop_id,
       g_object_set_property (G_OBJECT (ice->priv->nice_agent),
           "controlling-mode", value);
       break;
+    case PROP_FORCE_RELAY:
+      g_object_set_property (G_OBJECT (ice->priv->nice_agent),
+          "force-relay", value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -858,6 +863,10 @@ gst_webrtc_ice_get_property (GObject * object, guint prop_id,
     case PROP_AGENT:
       g_value_set_object (value, ice->priv->nice_agent);
       break;
+    case PROP_FORCE_RELAY:
+      g_object_get_property (G_OBJECT (ice->priv->nice_agent),
+          "force-relay", value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -924,6 +933,12 @@ gst_webrtc_ice_class_init (GstWebRTCICEClass * klass)
           "ICE agent in use by this object", NICE_TYPE_AGENT,
           G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class,
+      PROP_FORCE_RELAY,
+      g_param_spec_boolean ("force-relay", "Force Relay",
+          "Force all traffic to go through a relay.", FALSE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GstWebRTCICE::on-ice-candidate:
    * @object: the #GstWebRtcBin
