@@ -62,6 +62,11 @@ static const struct fourcc_map gst_msdk_fourcc_mfx_to_va[] = {
   FOURCC_MFX_TO_VA (RGB4, ARGB),
   FOURCC_MFX_TO_VA (P8, P208),
   FOURCC_MFX_TO_VA (P010, P010),
+#if (MFX_VERSION >= 1028)
+  FOURCC_MFX_TO_VA (RGB565, RGB565),
+#endif
+  FOURCC_MFX_TO_VA (AYUV, AYUV),
+  FOURCC_MFX_TO_VA (A2RGB10, A2R10G10B10),
   {0, 0}
 };
 
