@@ -289,8 +289,9 @@ gst_shm_sink_allocator_alloc_locked (GstShmSinkAllocator * self, gsize size,
     if (padding && (params->flags & GST_MEMORY_FLAG_ZERO_PADDED))
       memset (mymem->data + params->prefix + size, 0, padding);
 
-    gst_memory_init (memory, params->flags, g_object_ref (self), NULL,
-        maxsize, align, params->prefix, size);
+    gst_memory_init (memory, params->flags,
+        GST_ALLOCATOR_CAST (g_object_ref (self)), NULL, maxsize, align,
+        params->prefix, size);
   }
 
   return memory;
@@ -351,6 +352,7 @@ gst_shm_sink_init (GstShmSink * self)
 {
   g_cond_init (&self->cond);
   self->size = DEFAULT_SIZE;
+  self->unlock = FALSE;
   self->wait_for_connection = DEFAULT_WAIT_FOR_CONNECTION;
   self->perms = DEFAULT_PERMS;
 
@@ -669,6 +671,11 @@ gst_shm_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   gsize written_bytes;
 
   GST_OBJECT_LOCK (self);
+  if (self->unlock) {
+    GST_OBJECT_UNLOCK (self);
+    return GST_FLOW_FLUSHING;
+  }
+
   while (self->wait_for_connection && !self->clients) {
     g_cond_wait (&self->cond, GST_OBJECT_GET_LOCK (self));
     if (self->unlock) {
